import java.util.List;
import java.util.Iterator;
public class Tree<Label> implements Iterable<Label>{
    public Label label;        
    public Tree[] children;

    public Tree(Label x) {
        label = x;
        children = new Tree[0];
    }


    public Label label() {
        return label;
    }

    public int numChildren() {
        return children.length;
    }

    public Tree child(int k) {
        return children[k];
    }

    public void preorderTraverse(Tree<Label> T, Action<Label> whatToDo)
    {
        if (T != null) {
            whatToDo.action (T);
            for (int i = 0; i < T.numChildren (); i += 1)
                preorderTraverse (T.child (i), whatToDo);
        }
    }

    public void preorderTraverseIterative(Tree<Label> T, Action<Label> whatToDo)
    {
        Stack<Tree<Label>> s = new Stack<Tree<Label>>();
        s.push(T);
        while (!s.isEmpty()) {
            Tree<Label> node = s.pop();
            if (node == null)
                continue;

            whatToDo.action (node);
            for (int i = node.numChildren()-1; i >= 0; i -= 1)
                s.push(node.child(i));
        }
    }

    private class preorderIterator implements Iterator<Label>{
        Stack<Tree<Label>> s = new Stack<Tree<Label>>();        
        /** Can't push 'this' since 'this' is an iterator. 
            To get the Tree that 'this' is a baby of, use Tree.this.
            Yes, this is new syntax. */
        public preorderIterator() {
            s.push(Tree.this); /* new syntax, Tree.this is parent tree */
        }
        public boolean hasNext() {
            return (!s.isEmpty());
        }
        public Label next() {
            Tree<Label> node = s.pop();
            for (int i = node.numChildren()-1; i >= 0; i -= 1)
                s.push(node.child(i));
            return node.label;           
        }
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }

    public Iterator<Label> iterator() {
        return new preorderIterator();
    }

    public Iterator<Label> jankyIterator(Tree<Label> T) {
        ListBuilder<Label> lb = new ListBuilder<Label>();
        T.preorderTraverse(T, lb);
        return lb.L.iterator();
    }


    public static void main(String[] args) {
        Tree<String> someTree = new Tree<String>("D");
        someTree.children = new Tree[2];
        someTree.children[0] = new Tree<String>("B");
        someTree.children[1] = new Tree<String>("F");
        someTree.children[0].children = new Tree[2];
        someTree.children[0].children[0] = new Tree<String>("A");
        someTree.children[0].children[1] = new Tree<String>("C");

        someTree.children[1].children = new Tree[3];
        someTree.children[1].children[0] = new Tree<String>("E");
        someTree.children[1].children[1] = new Tree<String>("G");
        someTree.children[1].children[2] = new Tree<String>("H");



        /** awkward to invoke via an instance tree, but given that
        the method is not static, we can't help it. ordinarily,
        we wouldn't be operating on trees directly (we'd be using
        sets, maps, or whatever), so this problem would be avoided.

        Why not just make it static? Because we're using generics.
        More next week. */ 
        ListBuilder<String> lb = new ListBuilder<String>();
        someTree.preorderTraverse(someTree, lb);
        for (String s : lb.L) {
            System.out.println(s);
        }

        for (String s : someTree) {
            System.out.println(s);
        }
    }
}